from rocolib.api.components.Component import Component

c = Component()

#Parameters for the sheath that would cover the wheels and the ESP32 Stack
#Parameters that will change the dimension of the car
c.addParameter("length", 65, paramType="length", minValue=60) #tested minValue
c.addParameter("height", 48, paramType="length", minValue=36) #tested minValue
c.addParameter("width", 60, paramType="length", minValue=51) #tested minValue

#This car will use the customized ESP32 stack in dimensions.py and the FS90R servo
#Change here if you have another board
c.addParameter("driveservo", "fs90r", paramType="dimension")
c.addParameter("brains", "esp32stack", paramType="dimension")

#Four wheels using Wheel.yaml
c.addSubcomponent("fleft", "Wheel", invert=True, inherit="angle")
c.addSubcomponent("bright", "Wheel", invert=True, inherit="angle")

#Setting the parameters for the Wheels
for servo in ("bright", "fleft"):
    c.addConstraint((servo, "servo"), "driveservo")
    c.addConstraint((servo, "length"), ("length", "brains"), '((x[0] - getDim(x[1], "width")))')
    c.addConstConstraint((servo, "center"), False)
    c.addConstraint((servo, "radius"), "brains", '(getDim(x, "height") / 1.35)')

#Sheath to cover the wheels and the ESP32 Stack
c.addSubcomponent("sheath1", "SimpleRectBeam")
c.addConstraint(("sheath1", "length"), "length")
c.addConstraint(("sheath1", "depth"), "height")
c.addConstraint(("sheath1", "width"), "width")

#Support for the middle of the front and back wheels so that they won't collapse or wiggle
for i in range(1,3):
    c.addSubcomponent("between%d" %i, "SimpleRectBeam")
    c.addConstraint(("between%d" %i, "length"), ("width", "driveservo"), "x[0]- 2 * getDim(x[1], 'motorheight')")
    c.addConstraint(("between%d" % i, "depth"), "driveservo", "getDim(x, 'motorwidth')")
    c.addConstraint(("between%d" % i, "width"), "driveservo", "getDim(x, 'motorwidth')")

#Back between support
c.addConnection(("fleft", "topedge2"), ("between1", "topedge1"), angle=90)
c.addConnection(("between1", "botedge1"), ("bright", "botedge2"), angle=90)
c.addConnection(("between2", "topedge1"), ("bright", "topedge2"), angle=90)
c.addConnection(("between2", "botedge1"), ("fleft", "botedge2"), angle=90, tabWidth=10)

#Flipping he front left and right wheels to match the orientation of the
#bottom left and right wheels
c.addConstConstraint(("bright", "flip"), True)

# #Connecting the sheath to the ESP32 stack and wheels
for i in range(2):
    c.addSubcomponent("sheathsplit%d" %i, "SplitEdge")
    c.addConstraint(("sheathsplit%d" %i, "toplength"), "height", "(x,)")

c.addConstraint(("sheathsplit0", "botlength"), ("driveservo", "height"),
                "(getDim(x[0],'motorwidth'), x[1] - getDim(x[0],'motorwidth'))")

c.addConstraint(("sheathsplit1", "botlength"), ("driveservo", "height"),
                "(x[1] - getDim(x[0],'motorwidth'), getDim(x[0],'motorwidth'))")

c.addConnection(("sheathsplit0", "topedge0"), ("sheath1", "topedge1"))
c.addConnection(("sheathsplit1", "topedge0"), ("sheath1", "topedge3"))
c.addConnection(("fleft", "botedge0"), ("sheathsplit0", "botedge0"), angle=180)

c.inheritAllInterfaces("sheath1")
c.inheritInterface("splitedge0", ("sheathsplit0", "botedge1"))
c.inheritInterface("splitedge1", ("sheathsplit1", "botedge0"))

c.toLibrary("TwoWheels")
