from rocolib.api.components import DecorationComponent
from rocolib.api.composables.graph.Face import Face


class Header(DecorationComponent):
  def define(self):
      self.addParameter("nrows", 3, paramType="count")
      self.addParameter("ncols", 1, paramType="count")
      self.addParameter("rowsep", 2.54, paramType="length")
      self.addParameter("colsep", 2.54, paramType="length")
      self.addParameter("diameter", 1, paramType="length")

  def assemble(self):
    diam = self.getParameter("diameter")/2.
    nr = self.getParameter("nrows")
    nc = self.getParameter("ncols")

    def hole(i, j, d):
      dx = (j - (nc-1)/2.)*self.getParameter("colsep")
      dy = (i - (nr-1)/2.)*self.getParameter("rowsep")
      return Face("r-%d-%d" % (i,j),
                        ((dx-d, dy-d), (dx+d, dy-d), (dx+d, dy+d), (dx-d, dy+d)),
                        recenter=False)

    for i in range(nr):
      for j in range(nc):
        d = diam
        if (i == 0 and j == 0) or \
           (i == nr-1 and j == nc-1):
               d = diam*3
        self.addFace(hole(i,j,d), prefix="r-%d-%d" % (i,j))

if __name__ == "__main__":
    Header.test()
